; Gameboy Frogs and Flies Sample Program by Charles Doty (cdoty@pcisys.net)

    .include "defines.inc"
    .include "palette.inc"
    .include "routines.inc"

    Transfer = RAM_START+193; to RAM_START+194

    .module MAIN

    .area _CODE1 (REL)

start::
    ld    sp, #0xe000           ; Put the stack where the GB wants it

    ld    a, #0x00              ; No IRQs at all
    ldh   (0xff), a

    sub   a                     ; Misc standard init things..
    ldh   (0x41), a             ; LCDC Status
    ldh   (0x42), a             ; Screen scroll Y=0
    ldh   (0x43), a             ; Screen scroll X=0

    ld    a, #0x07              ; LCD Controller = Off (No picture on screen)
                                ; WindowBank = 0x9800 (Not used)
                                ; Window = OFF
                                ; BG Chr = 0x8800
                                ; BG Bank= 0x9800
                                ; OBJ    = 8x16
                                ; OBJ    = On
                                ; BG     = On
          
    call  ScreenOff

    ld    hl, #0x8000
    ld    bc, #0x1800

    call  ClearVRAM

    xor   a                     ; Select palette index 0
    ld    hl, #bkgpal           ; Set up hl to point to background palette
    ld    b, #bkgpalsize        ; Set palette size

    call  TransferBkgPalette    ; Transfer palette

    xor   a                     ; Select palette index 0
    ld    hl, #sprpal           ; Set up hl to point to background palette
    ld    b, #sprpalsize        ; Set palette size

    call  TransferSprPalette    ; Normal palette

    xor   a                     ; Use bank 0 (map) of map location
    ldh   (0x4f), a

    ld    bc, #TilesLen
    ld    de, #TILE_LOCATION
    ld    hl, #Tiles

    call  TransferCharData      ; Move the charset to TILE_LOCATION

    ld    bc, #SpritesLen
    ld    de, #SPR_LOCATION
    ld    hl, #Sprites

    call  TransferCharData      ; Move the sprites to SPR_LOCATION

    ld    a, #1                 ; Use bank 1 (attributes) of map location
    ldh   (0xff4f), a

    ld    bc, #AttributesLen
    ld    de, #MAP_LOCATION
    ld    hl, #Attributes

    call  TransferCharData      ; Move the attributes to TILE_LOCATION
                                ; (Bank 1)

    xor   a                     ; Use bank 0 (map) of map location
    ldh   (0x4f), a

    ld    bc, #MapLen
    ld    de, #MAP_LOCATION
    ld    hl, #Map

    call  TransferCharData      ; Move the charset to TILE_LOCATION

    ld    hl, #ListData         ; Set the location of the sprite list
    ld    a, #ListCount/3       ; Load a with the number of sprites in the list

    call  SetSpriteList
    call  Update

    ld    a, #0x87              ; LCD Controller = On

    call  ScreenOnVB
    ei

EndlessLoop:
    ld    a, #13
    ld    b, #102
    ld    c, #32
          
    call  ChangeFrogChar
    call  Update
    call  WaitForTransfer
    
    ld    a, #13
    ld    b, #104
    ld    c, #32

    call  ChangeFrogChar
    call  Update
    call  WaitForTransfer

    ld    a, #13
    ld    b, #106
    ld    c, #32

    call  ChangeFrogChar
    call  Update
    call  WaitForTransfer

    ld    a, #13
    ld    b, #104
    ld    c, #32

    call  ChangeFrogChar
    call  Update
    call  WaitForTransfer

    jp    EndlessLoop

; Changes frog sprite.
; Load A with sprite to start with
; Load B with tongue sprite
; Load C with tile to start with
ChangeFrogChar:
    push  af
    push  bc
    push  de

    call  SetSpriteChar
    inc   a
    ld    b, c

    ld    e, #8

SpriteLoop1:
    call  SetSpriteChar

    inc   a
    inc   b
    inc   b

    ld    d, a
    ld    a, e
    dec   a

    ld    e, a

    or    a
    ld    a, d

    jr    nz, SpriteLoop1

    pop   de
    pop   bc
    pop   af

    ret
    
WaitForTransfer:
    ld    a, (Transfer)
    or    a
    jr    nz, WaitForTransfer

    ret

Tiles:
    INCBIN "tiles.bin"
TilesEnd:
TilesLen = TilesEnd - Tiles

Attributes:
    INCBIN "attrib.bin"
AttributesEnd:
AttributesLen = AttributesEnd - Attributes

Sprites:
    INCBIN "sprites.bin"
SpritesEnd:
SpritesLen = SpritesEnd - Sprites

Map:
    INCBIN "map.bin"
MapEnd:
MapLen = MapEnd - Map

ListData:
    .byte  24,  42, 108,  32,  42, 110 
    .byte 112,  42, 108, 120,  42, 110
    .byte  36, 112,  00
    .byte  10,  96,  64,  18,  96,  66,  26,  96,  68,  34,  96, 70
    .byte  10, 112,  72,  18, 112,  74,  26, 112,  76,  34, 112, 78
    .byte 132, 112,  00
    .byte 134,  96,  00, 142,  96,  02, 150,  96,  04, 158,  96, 06
    .byte 134, 112,  08, 142, 112,  10, 150, 112,  12, 158, 112, 14
ListEnd:
ListCount = ListEnd-ListData
