; Gameboy Frogs and Flies Sample Program by Charles Doty (cdoty@pcisys.net)

    INCLUDE "defines.inc"
    INCLUDE "routines.inc"

    SECTION "Code",HOME[$0150]

start::
    call SetDoubleSpeedMode     ; Puts the gameboy in double speed mode
    ld   sp, $E000              ; Put the stack where the GB wants it

    sub  a                      ; No IRQs at all
    ldh  [$ff], a

    ldh  [$41], a               ; LCDC Status
    ldh  [$42], a               ; Screen scroll Y=0
    ldh  [$43], a               ; Screen scroll X=0

    ld   a, $07                 ; LCD Controller = Off (No picture on screen)
                                ; WindowBank = $9800 (Not used)
                                ; Window = OFF
                                ; BG Chr = $8800
                                ; BG Bank= $9800
                                ; OBJ    = 8x16
                                ; OBJ    = On
                                ; BG     = On
    call ScreenOff

    ld   hl, $8000
    ld   bc, $1800

    call ClearVRAM

    xor  a                      ; Select palette index 0
    ld   hl, TilesPal           ; Set up hl to point to background palette
    ld   b, TilesPalEnd-TilesPal; Set palette size

    call TransferBkgPalette     ; Transfer palette

    xor  a                      ; Select palette index 0
    ld   hl, SprPal             ; Set up hl to point to background palette
    ld   b, SprPalEnd-SprPal    ; Set palette size

    call TransferSprPalette     ; Normal palette

    xor  a                      ; Use bank 0 (map) of map location
    ldh  [$4f], a

    ld   bc, TilesEnd-Tiles
    ld   de, TILE_LOCATION
    ld   hl, Tiles

    call TransferCharData       ; Move the charset to TILE_LOCATION

    ld   bc, SpritesEnd-Sprites
    ld   de, SPR_LOCATION
    ld   hl, Sprites

    call TransferCharData       ; Move the sprites to SPR_LOCATION

    ld   a, 1                   ; Use bank 1 (attributes) of map location
    ldh  [$4f], a

    ld   bc, AttributesEnd-Attributes
    ld   de, MAP_LOCATION
    ld   hl, Attributes

    call TransferCharData       ; Move the attributes to TILE_LOCATION
                                ; (Bank 1)
    xor  a                      ; Use bank 0 (map) of map location
    ldh  [$4f], a

    ld   bc, MapEnd-Map
    ld   de, MAP_LOCATION
    ld   hl, Map

    call TransferCharData       ; Move the charset to TILE_LOCATION

    ld   hl, List               ; Set the location of the sprite list
    ld   a, (ListEnd-List)/3    ; Load a with the number of sprites in the list

    call SetSpriteList
    call Update

    ld   a, $87                 ; LCD Controller = On

    call ScreenOnVB
    ei

EndlessLoop:
    ld   a, 13
    ld   b, 102
    ld   c, 32
          
    call ChangeFrogChar
    call Update
    call WaitForTransfer
    
    ld   a, 13
    ld   b, 104
    ld   c, 32

    call ChangeFrogChar
    call Update
    call WaitForTransfer

    ld   a, 13
    ld   b, 106
    ld   c, 32

    call ChangeFrogChar
    call Update
    call WaitForTransfer

    ld   a, 13
    ld   b, 104
    ld   c, 32

    call ChangeFrogChar
    call Update
    call WaitForTransfer

    jp   EndlessLoop

; Changes frog sprite.
; Load A with sprite to start with
; Load B with tongue sprite
; Load C with tile to start with
ChangeFrogChar:
    push af
    push bc
    push de

    call SetSpriteChar
    inc  a
    ld   b, c

    ld   e, 8

SpriteLoop1:
    call SetSpriteChar

    inc  a
    inc  b
    inc  b

    ld   d, a
    ld   a, e
    dec  a

    ld   e, a

    or   a
    ld   a, d

    jr   nz, SpriteLoop1

    pop  de
    pop  bc
    pop  af

    ret
    
WaitForTransfer:
    ld   a, [Transfer]
    or   a
    jr   nz, WaitForTransfer

    ret

Tiles:
    INCBIN "tiles.bin"
TilesEnd:

Attributes:
    INCBIN "attrib.bin"
AttributesEnd:

Sprites:
    INCBIN "sprites.bin"
SpritesEnd:

Map:
    INCBIN "map.bin"
MapEnd:

List:
    DB  24,  42, 108,  32,  42, 110 
    DB 112,  42, 108, 120,  42, 110
    DB  36, 112,  00
    DB  10,  96,  64,  18,  96,  66,  26,  96,  68,  34,  96, 70
    DB  10, 112,  72,  18, 112,  74,  26, 112,  76,  34, 112, 78
    DB 132, 112,  00
    DB 134,  96,  00, 142,  96,  02, 150,  96,  04, 158,  96, 06
    DB 134, 112,  08, 142, 112,  10, 150, 112,  12, 158, 112, 14
ListEnd:

TilesPal:
    INCBIN "tilespal.bin"
TilesPalEnd:

SprPal:
    INCBIN "sprpal.bin"
SprPalEnd:
