; Gameboy Routines

    .include "defines.inc"
    .include "sprite.inc"

    SpriteY  = RAM_START    ; to RAM_START+64
    SpriteX  = RAM_START+64 ; to RAM_START+192 - Contains Sprite X and characters interleaved
    Joystick = RAM_START+192; to RAM_START+193
    Transfer = RAM_START+193; to RAM_START+194

    .area _CODE2 (REL)
    .module ROUTINES

ClearVRAM::
    push  af
    push  de

; Clears VRAM. See routines.inc for full description
ClearVRAMLoop:

    sub   a                 ; Clear a
    ld    (hl), a
    inc   hl
    dec   bc                ; decrement counter
    ld    a, b              ; transfer upper byte of counter to a
    or    c                 ; or with c. Check to see if both are zero
    jr    nz, ClearVRAMLoop ; If not zero repeat

    ld    hl, #0xfe00
    ld    a,  #0xa0

ClearSprites:
    ld    (hl), #0          ; Transfer byte to VRAM
    inc   hl

    dec   a                 ; decrement counter
    or    a                 ; or with c. Check to see if both are zero
    jr    nz, ClearSprites  ; If not zero repeat

    ld    hl, #OAM_LOCATION
    ld    a,  #0xa0

ClearSpriteMem:
    ld    (hl), #0          ; Transfer byte to VRAM
    inc   hl

    dec   a                 ; decrement counter
    or    a                 ; or with c. Check to see if both are zero
    jr    nz, ClearSpriteMem; If not zero repeat

    sub   a
    ld    (Transfer), a

    ld    hl, #SPRITE_ROUTINE
    ld    de, #SpriteRoutine
    ld    b,  #SpriteRoutineSize

RoutineLoop:
    ld    a, (de)
    ld    (hl), a           ; Transfer byte to SPRITE_ROUTINE

    inc   de
    inc   hl

    ld    a, b
    dec   a                 ; decrement counter
    or    a                 ; or with c. Check to see if both are zero
    ld    b, a

    jr    nz, RoutineLoop   ; If not zero repeat

    pop   de
    pop   af

    ret

; Transfer Palette to VRAM. See routines.inc for full description
TransferBkgPalette::
    sla   a  
    or    a, #0x80          ; Sets up to write to palette 0 color 0, auto
    ldh   (0x68), a         ; increment Store in Bkg Write Specification
                            ; register
ploop1:
    ld    a, (hl)           ; Store low byte, then high byte in 
    ldh   (0x69), a         ; Background Write Register
    inc   hl

    dec   b
    ld    a, b
    or    a
    jr    nz, ploop1

    ld    a, #0xe4          ; grey 3=11 (Black)
                            ; grey 2=10 (Dark grey)
                            ; grey 1=01 (Light grey)
                            ; grey 0=00 (Black/Transparent)
    ldh   (0x47), a

    ret

; Transfer Palette to VRAM. See routines.inc for full description
TransferSprPalette::
    sla   a  
    or    a, #0x80          ; Sets up to write to palette 0 color 0, auto 
    ldh   (0x6a), a         ; increment Store in Spr Write Specification
                            ; register
ploop2:
    ld    a, (hl)           ; Store low byte, then high byte in 
    ldh   (0x6b), a         ; Sprite Write Register
    inc   hl

    dec   b
    ld    a, b
    or    a
    jr    nz, ploop2

    ld    a, #0xe4          ; grey 3=11 (Black)
                            ; grey 2=10 (Dark grey)
                            ; grey 1=01 (Light grey)
                            ; grey 0=00 (Black/Transparent)
    ldh   (0x48), a
    ldh   (0x49), a

    ret

; Transfer Character data to VRAM. See routines.inc for full description
TransferCharData::
    push af

CharLoop:
    ld   a, (hl)            ; Load char data
    ld   (de), a
    inc  hl                 ; increment memory location
    inc  de                 ; increment memory location
    dec  bc                 ; decrement counter
    ld   a, b               ; transfer upper byte of counter to a
    or   c                  ; or with c. Check to see if both are zero
    jr   nz, CharLoop       ; If not zero repeat

    pop  af

    ret

; Turn on screen. See routines.inc for full description
ScreenOn::
    ldh   (0x40), a

    ret

; Turn on screen with vertical blank. See routines.inc for full description
ScreenOnVB::
    push af

    ldh   (0x40), a

    ldh  a, (0x41)
    or   #0x10
    ldh   (0x41), a

    ldh  a, (0x0f)
    or   #1
    ldh  (0x0f), a

    ld   a, #0x01
    ldh  (0xff), a

    pop  af

    ret

; Turn off screen. See routines.inc for full description
ScreenOff::
    push af
    push bc

    di

ScOff:
    ld   b, a
    ldh  a, (0x44)
    cp   #0x90
    jr   nz, ScOff
    xor  a

    ld   a, b
    ldh  (0x40), a

    ei

    pop  bc
    pop  af

    ret

; Set sprite x, y, and character from a memory location.
; See routines.inc for full description
SetSpriteList::
    push bc
    push de

    push af
    sub  a

ListLoop:
    ld   b, (hl)
    inc  hl
    ld   c, (hl)
    inc  hl   
    ld   d, (hl)
    inc  hl   
        
    call SetSprite
    inc  a

    ld   e, a

    pop  af
    dec  a
    push af

    or   #0
    ld   a, e
    
    jr   nz, ListLoop       ; If not zero repeat

    pop  af
    pop  de
    pop  bc
    ret

; Set sprite x, y, and character. See routines.inc for full description
SetSprite::
    push bc

    call SetSpriteX         ; Already set up for SetSpriteX. Call routine
    ld   b, c               ; Transfer y position into b
    call SetSpriteY         ; Call routine
    ld   b, d               ; Transfer character number into b
    call SetSpriteChar      ; Call routine

    pop  bc

    ret

; Set sprite x. See routines.inc for full description
SetSpriteX::
    push af
    push bc
    push hl

    add  a, a           ; Multiplies a by 4
    add  a, a
    inc  a              ; Adjust a to point to X member

    ld   hl, #0xc000

    ld   l, a

    ld   (hl), b        ; Transfer x position

    pop  hl
    pop  bc
    pop  af

    ret

; Set sprite y. See routines.inc for full description
SetSpriteY::
    push af
    push bc
    push hl

    add  a, a           ; Multiplies a by 4
    add  a, a

    ld   hl, #0xc000

    ld   l, a

    ld   (hl), b        ; Transfer y position

    pop  hl
    pop  bc
    pop  af

    ret

; Set sprite character. See routines.inc for full description
SetSpriteChar::
    push af
    push bc
    push hl

    add  a, a               ; Multiplies a by 4
    add  a, a
    inc  a                  ; Adjust a to point to character member
    inc  a                  ; Adjust a to point to character member

    ld   hl, #OAM_LOCATION

    ld   l, a

    ld   (hl), b            ; Transfer character number

    push hl
    ld   hl, #sprattrib
    ld   a, b
    srl  a
    add  a, l
    ld   l, a
    
    ld   a, (hl)
    pop  hl

    inc  hl

    ld   (hl), a

    pop  hl
    pop  bc
    pop  af

    ret

; Set sprite palette. See routines.inc for full description
SetSpritePal::
    push af
    push bc
    push hl

    add  a, a               ; Multiplies a by 4
    add  a, a
    inc  a                  ; Adjust a to point to character member
    inc  a                  ; Adjust a to point to character member
    inc  a                  ; Adjust a to point to character member

    ld   hl, #OAM_LOCATION

    ld   l, a

    ld   (hl), b            ; Transfer character number

    pop  hl
    pop  bc
    pop  af

    ret

Update::
    ld  a, #1
    ld  (Transfer), a

    ret

VertBlank::
    push af
    push bc
    push de
    push hl

    call ReadJoystick
    call TransferSprites

    pop  hl
    pop  de
    pop  bc
    pop  af

    ei
    reti

ReadJoystick:
    ld   a, #0x10
    ldh  (0xf0), a
    ldh  a, (0xf0)
    ldh  a, (0xf0)

    and  #0x0f
    swap  a
    ld    b, a
    ld    a, #0x20
    ldh   (0xf0), a
    ldh   a, (0xf0)
    ldh   a, (0xf0)
    ldh   a, (0xf0)
    ldh   a, (0xf0)
    ldh   a, (0xf0)
    ldh   a, (0xf0)

    and   #0x0f
    or    b
    cpl

    ld   b, a
    or   #0x3f

    ld   (Joystick), a

    ld   a, #0x30
    ldh  (0xf0), a

    ret

TransferSprites:
    ld   a, (Transfer)
    or   a
    jr   z, NoTransfer      ; If not zero repeat    

    jp   SPRITE_ROUTINE

NoTransfer:
    ret

; Routine that is transfered to SPRITE_ROUTINE
SpriteRoutine:
    push af
    push hl

    ld   hl, #OAM_LOCATION
    ld   a, h
    ldh  (0x46), a

    ld   a, #0x28

Wait:
    dec  a
    jr   nz, Wait

    sub  a
    ld   (Transfer), a

    pop  hl
    pop  af

    ret
SpriteRoutineEnd:
SpriteRoutineSize = SpriteRoutineEnd - SpriteRoutine
